"""Lazy import wrapper for the tree motion adapter.

Accessing attributes of this module triggers loading of the optional
``arbolab_treemotion`` package.  If it is not installed a clear error is
raised guiding the user to install the ``treemotion`` extra.
"""

from __future__ import annotations

from ..utils.lazy_import import lazy_module

__getattr__ = lazy_module("arbolab_treemotion", extra="treemotion")

__all__ = ["__getattr__"]



from pathlib import Path

import pytest

from arbolab.utils.path import WorkDir, ensure_dir


def test_workdir_creates_subdirectories(tmp_path: Path) -> None:
    wd = WorkDir(tmp_path)
    assert wd.logs.is_dir()
    assert wd.plots.is_dir()
    assert wd.data.is_dir()
    assert wd.db.is_dir()
    db_file = wd.db_path()
    assert db_file.parent == wd.db


def test_workdir_raises_without_root() -> None:
    wd = WorkDir(None)
    with pytest.raises(ValueError):
        _ = wd.logs


def test_ensure_dir_none_raises_typeerror() -> None:
    with pytest.raises(TypeError):
        ensure_dir(None)



from pathlib import Path

import pytest

from arbolab.classes.project import Project
from arbolab.config import Config, DBConfig, PlotConfig, LoggingConfig
from arbolab.lab import Lab


def test_lab_setup_and_load(tmp_path: Path) -> None:
    cfg = Config(
        working_dir=tmp_path,
        logging=LoggingConfig(level="DEBUG"),
        db=DBConfig(echo=True),
        plot=PlotConfig(backend="Agg"),
    )
    lab = Lab.setup(cfg, load_plugins=False)
    Project.setup(lab, "demo")
    db_file = Path(lab.config.db_url.split("///", 1)[1])

    loaded = Lab.load(db_file, load_plugins=False)
    persisted = loaded.config_store.load()
    assert persisted == loaded.config
    assert loaded.config.log_level == "DEBUG"
    assert loaded.config.db.echo is True
    assert loaded.config.plot.backend == "Agg"
    project = Project.load(loaded, "demo")
    assert project.name == "demo"


def test_lab_load_missing_file(tmp_path: Path) -> None:
    missing = tmp_path / "missing.duckdb"
    with pytest.raises(FileNotFoundError):
        Lab.load(missing, load_plugins=False)



"""Database utilities for arbolab.

This package provides a thin abstraction over SQLAlchemy's session handling to
make database access convenient in interactive environments while remaining
fully compatible with more advanced use cases such as web applications.
"""

from .manager import DBManager
from .session import (
    current_session,
    make_session_factory,
    session_scope,
    with_session,
)

__all__ = [
    "DBManager",
    "make_session_factory",
    "session_scope",
    "current_session",
    "with_session",
]



"""Enumeration of lifecycle statuses for entities."""

from __future__ import annotations

from enum import StrEnum


class Status(StrEnum):
    """Lifecycle status for entities."""

    DRAFT = "draft"
    ACTIVE = "active"
    ARCHIVED = "archived"
    DELETED = "deleted"



import pandas as pd
import pytest

from arbolab.utils import (
    calc_amplitude,
    calc_min_max,
    calc_sample_rate,
    time_cut_by_datetime_index,
    validate_df,
    validate_time_format,
)


def test_calc_sample_rate_datetime_index() -> None:
    idx = pd.date_range("2023-01-01", periods=4, freq="1s")
    df = pd.DataFrame({"value": [1, 2, 3, 4]}, index=idx)
    assert calc_sample_rate(df) == pytest.approx(1.0)


def test_calc_sample_rate_time_column() -> None:
    df = pd.DataFrame({"time": [0.0, 0.5, 1.0], "val": [0, 1, 2]})
    assert calc_sample_rate(df, "time") == pytest.approx(2.0)


def test_calc_amplitude_and_min_max() -> None:
    df = pd.DataFrame({"t": [0, 1, 2], "s": [1, 3, 2]})
    assert calc_amplitude(df, "s") == 1.0
    result = calc_min_max(df, "s", "t")
    assert result["max"]["value"] == 3
    assert result["max"]["time"] == 1
    assert result["min"]["value"] == 1
    assert result["min"]["time"] == 0


def test_validate_time_format_and_cut() -> None:
    df = pd.DataFrame(
        {"a": [1, 2, 3]},
        index=pd.date_range("2024-01-01 00:00:00", periods=3, freq="1s"),
    )
    start = "2024-01-01 00:00:01"
    end = "2024-01-01 00:00:02"
    limited = time_cut_by_datetime_index(df, start, end)
    assert len(limited) == 2
    formatted = validate_time_format(start)
    assert formatted == "2024-01-01 00:00:01.000000"


def test_validate_df() -> None:
    df = pd.DataFrame({"a": [1, 2], "b": [3, 4]})
    assert validate_df(df, ["a"])
    with pytest.raises(TypeError):
        validate_df([1, 2])  # type: ignore[arg-type]
    with pytest.raises(ValueError):
        validate_df(pd.DataFrame(), ["a"])
    with pytest.raises(ValueError):
        validate_df(pd.DataFrame({"a": [1, None]}))
    with pytest.raises(ValueError):
        validate_df(df, ["c"])
